/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceAccessorCatalog_h__
#define __surface_SurfaceAccessorCatalog_h__

namespace fe
{

/**************************************************************************//**
    @brief Accessor backed with a Catalog

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessorCatalog:
	virtual public SurfaceAccessorBase
{
	public:
						SurfaceAccessorCatalog(void);
virtual					~SurfaceAccessorCatalog(void);

						//* as SurfaceAccessorI
virtual	U32				count(void) const
						{
							switch(m_attrType)
							{
								case e_string:
									return m_spCatalog->catalog<
											std::vector<String> >(
											m_key,m_attrName).size();
								case e_integer:
									return m_spCatalog->catalog<
											std::vector<I32> >(
											m_key,m_attrName).size();
								case e_real:
									return m_spCatalog->catalog<
											std::vector<Real> >(
											m_key,m_attrName).size();
								case e_spatialVector:
									return m_spCatalog->catalog<
											std::vector<SpatialVector> >(
											m_key,m_attrName).size();
								default:
									;
							}
							return 0;
						}

virtual	void			set(U32 a_index,U32 subIndex,String a_string)
						{
							std::vector<String>& rStringList=
									m_spCatalog->catalog<std::vector<String> >(
									m_key,m_attrName);

							const U32 size=rStringList.size();
							rStringList.resize(a_index+1);
							rStringList[a_index]=a_string;

							for(U32 index=size;index<a_index;index++)
							{
								rStringList[index]="";
							}
						}
virtual	String			string(U32 a_index,U32 a_subIndex=0)
						{
							const std::vector<String>& rStringList=
									m_spCatalog->catalog<std::vector<String> >(
									m_key,m_attrName);
							if(a_index<rStringList.size())
							{
								return rStringList[a_index];
							}

							return "";
						}

virtual	void			set(U32 a_index,U32 a_subIndex,I32 a_integer)
						{
							std::vector<I32>& rIntegerList=
									m_spCatalog->catalog<std::vector<I32> >(
									m_key,m_attrName);

							const U32 size=rIntegerList.size();
							rIntegerList.resize(a_index+1);
							rIntegerList[a_index]=a_integer;

							for(U32 index=size;index<a_index;index++)
							{
								rIntegerList[index]=0;
							}
						}
virtual	I32				integer(U32 a_index,U32 a_subIndex=0)
						{
							const std::vector<I32>& rIntegerList=
									m_spCatalog->catalog<std::vector<I32> >(
									m_key,m_attrName);
							if(a_index<rIntegerList.size())
							{
								return rIntegerList[a_index];
							}

							return 0;
						}

virtual	void			set(U32 a_index,U32 a_subIndex,Real a_real)
						{
							std::vector<Real>& rRealList=
									m_spCatalog->catalog<std::vector<Real> >(
									m_key,m_attrName);

							const U32 size=rRealList.size();
							rRealList.resize(a_index+1);
							rRealList[a_index]=a_real;

							for(U32 index=size;index<a_index;index++)
							{
								rRealList[index]=0.0;
							}
						}
virtual	Real			real(U32 a_index,U32 a_subIndex=0)
						{
							const std::vector<Real>& rRealList=
									m_spCatalog->catalog<std::vector<Real> >(
									m_key,m_attrName);
							if(a_index<rRealList.size())
							{
								return rRealList[a_index];
							}

							return 0.0;
						}

virtual	void			set(U32 a_index,U32 a_subIndex,
								const SpatialVector& a_spatialVector)
						{
							std::vector<SpatialVector>& rVectorList=
									m_spCatalog->catalog<
									std::vector<SpatialVector> >(
									m_key,m_attrName);

							const U32 size=rVectorList.size();
							rVectorList.resize(a_index+1);
							rVectorList[a_index]=a_spatialVector;

							for(U32 index=size;index<a_index;index++)
							{
								fe::set(rVectorList[index]);
							}

							return;
						}
virtual	SpatialVector	spatialVector(U32 a_index,U32 a_subIndex=0)
						{
							const std::vector<SpatialVector>& rVectorList=
									m_spCatalog->catalog<
									std::vector<SpatialVector> >(
									m_key,m_attrName);
							if(a_index<rVectorList.size())
							{
								return rVectorList[a_index];
							}

							return SpatialVector(0.0,0.0,0.0);
						}

		void			setSurfaceAccessible(
								sp<SurfaceAccessibleI> a_spSurfaceAccessibleI)
						{	m_spSurfaceAccessible=a_spSurfaceAccessibleI; }

						//* Catalog specific
		void			setCatalog(sp<Catalog> a_spCatalog)
						{	m_spCatalog=a_spCatalog; }
		void			setKey(String a_key)
						{	m_key=a_key; }

	private:

virtual	BWORD			bindInternal(SurfaceAccessibleI::Element a_element,
								const String& a_name);

		enum	AttrType
				{
					e_none,
					e_string,
					e_integer,
					e_real,
					e_spatialVector
				};
		sp<TypeMaster>	typeMaster(void)
						{	return registry()->master()->typeMaster(); }

		sp<SurfaceAccessibleI>			m_spSurfaceAccessible;
		sp<Catalog>						m_spCatalog;
		String							m_key;

		AttrType						m_attrType;
};

} // namespace

#endif /* __surface_SurfaceAccessorCatalog_h__ */
