/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "plugin.pmh"

namespace fe
{

Catalog::Catalog(void):
	m_highest(0)
{
}

Catalog::~Catalog(void)
{
	catalogClear();
}

sp<TypeMaster> Catalog::typeMaster(void) const
{
	return registry()->master()->typeMaster();
}

void Catalog::catalogDump(void)
{
	feLog("Catalog:\n");
	for(std::map< String, InstanceMap >::iterator it1=m_instanceMapMap.begin();
			it1!=m_instanceMapMap.end();it1++)
	{
		for(std::map< String, Instance >::iterator it2=it1->second.begin();
				it2!=it1->second.end();it2++)
		{
			feLog(" \"%s\" \"%s\" 0x%x \"%s\"\n", it1->first.c_str(),
					it2->first.c_str(), it2->second.data(),
					c_print(it2->second));
		}
	}
	feLog("\n");
}

void Catalog::catalogKeys(std::vector<String> &a_keys)
{
	for(std::map< I32, String >::iterator it = m_orderMap.begin();
			it != m_orderMap.end(); it++)
	{
		a_keys.push_back(it->second);
	}
}

bool Catalog::catalogLookup(const String a_name, const String a_property,
	Instance &a_instance)
{
	std::map<String, InstanceMap>::iterator it1 = m_instanceMapMap.find(a_name);
	if(it1 == m_instanceMapMap.end())
	{
		return false;
	}
	std::map<String, Instance>::iterator it2 =
			m_instanceMapMap[a_name].find(a_property);
	if(it2 == m_instanceMapMap[a_name].end())
	{
		return false;
	}
	a_instance = it2->second;
	return true;
}

} /* namespace */
